from django.urls import path
from netbox.views.generic import ObjectChangeLogView
from . import models, views

urlpatterns = (
    path('static-routes/', views.StaticRouteListView.as_view(), name='staticroute_list'),
    path('static-routes/add/', views.StaticRouteEditView.as_view(), name='staticroute_add'),
    path('static-routes/edit/', views.StaticRouteBulkEditView.as_view(), name='staticroute_bulk_edit'),
    path('static-routes/delete/', views.StaticRouteBulkDeleteView.as_view(), name='staticroute_bulk_delete'),
    path('static-routes/<int:pk>', views.StaticRouteView.as_view(), name='staticroute'),
    path('static-routes/<int:pk>/edit/', views.StaticRouteEditView.as_view(), name='staticroute_edit'),
    path('static-routes/<int:pk>/delete/', views.StaticRouteDeleteView.as_view(), name='staticroute_delete'),
    path('static-routes/<int:pk>/changelog/', ObjectChangeLogView.as_view(), name='staticroute_changelog', kwargs={
        'model': models.StaticRoute
    }),
)



from netbox.api.routers import NetBoxRouter
from . import views

app_name = 'netbox_static_routes'

router = NetBoxRouter()
router.register('static-routes', views.StaticRouteViewSet)

urlpatterns = router.urls



from netbox.api.viewsets import NetBoxModelViewSet
from .. import filtersets, models
from .serializers import StaticRouteSerializer

class StaticRouteViewSet(NetBoxModelViewSet):
    queryset = models.StaticRoute.objects.prefetch_related('tags')
    serializer_class = StaticRouteSerializer
    


from netbox.filtersets import NetBoxModelFilterSet
from .models import StaticRoute

class StaticRouteFilterSet(NetBoxModelFilterSet):

    class Meta:
        model = StaticRoute
        fields = ('id', 'site', 'device', 'vrf', 'destination_prefix')

    def search(self, queryset, name, value):
        return queryset.filter(description__icontains=value)
        


from extras.plugins import PluginConfig
from .version import __version__

class NetBoxStaticRoutesConfig(PluginConfig):
    name = 'netbox_static_routes'
    verbose_name = 'Static Routes'
    description = 'Manage static routes in Netbox'
    version = __version__
    author = 'Joshua Parrish'
    author_email = 'jbparrish17@gmail.com'
    base_url = 'static-routes'
    required_settings = []
    min_version = '3.2.0'
    max_version = '3.2.99'
    default_settings = {}

config = NetBoxStaticRoutesConfig



from dcim.models import Site, Device
from ipam.models import Prefix, VRF
from django import forms
from netbox.forms import NetBoxModelForm, NetBoxModelFilterSetForm, NetBoxModelBulkEditForm
from .models import StaticRoute
from utilities.forms.fields import CommentField, DynamicModelChoiceField

class StaticRouteForm(NetBoxModelForm):
    site = DynamicModelChoiceField(
        queryset=Site.objects.all(),
        required=False
    )
    device = DynamicModelChoiceField(
        queryset=Device.objects.all(),
        query_params={
            'site_id': '$site'
        }
    )
    vrf = DynamicModelChoiceField(
        queryset=VRF.objects.all(),
        required=False
    )
    # need to be able to handle no VRF
    destination_prefix = DynamicModelChoiceField(
        queryset=Prefix.objects.all(),
        query_params={
            'vrf_id': '$vrf'
        }
    )
    comments = CommentField()

    class Meta:
        model = StaticRoute
        fields = ('site', 'device', 'vrf', 'destination_prefix', 'next_hop', 'distance', 'comments', 'tags')

class StaticRouteFilterForm(NetBoxModelFilterSetForm):
    model = StaticRoute
    static_route = forms.ModelMultipleChoiceField(
        queryset=StaticRoute.objects.all(),
        required=False
    )
    site = forms.ModelMultipleChoiceField(
        queryset=Site.objects.all(),
        required=False
    )
    device = forms.ModelMultipleChoiceField(
        queryset=Device.objects.all(),
        required=False
    )
    vrf = forms.ModelMultipleChoiceField(
        queryset=VRF.objects.all(),
        required=False
    )
    destination_prefix = forms.ModelMultipleChoiceField(
        queryset=Prefix.objects.all(),
        required=False
    )

class StaticRouteBulkEditForm(NetBoxModelBulkEditForm):
    model = StaticRoute
    pk = forms.ModelMultipleChoiceField(
        queryset=StaticRoute.objects.all(),
        widget=forms.MultipleHiddenInput
    )
    distance = forms.IntegerField(
        max_value=255,
        min_value=1,
        required=False
    )



from graphene import ObjectType
from netbox.graphql.types import NetBoxObjectType
from netbox.graphql.fields import ObjectField, ObjectListField
from . import filtersets, models


class StaticRouteType(NetBoxObjectType):

    class Meta:
        model = models.StaticRoute
        fields = '__all__'

class Query(ObjectType):
    static_route = ObjectField(StaticRouteType)
    static_route_list = ObjectListField(StaticRouteType)
    
schema = Query


import django_tables2 as tables
from netbox.tables import NetBoxTable, ChoiceFieldColumn
from .models import StaticRoute
from django.urls import path


class StaticRouteTable(NetBoxTable):
    destination_prefix = tables.Column(
        linkify=(
            ('plugins:netbox_static_routes:staticroute', {'pk': tables.A('pk')})
        )
    )
    next_hop = tables.Column()
    site = tables.Column(
        linkify=True
    )
    device = tables.Column(
        linkify=True
    )
    vrf = tables.Column(
        linkify=True
    )

    class Meta(NetBoxTable.Meta):
        model = StaticRoute
        fields = ('pk', 'id', 'site', 'device', 'vrf', 'destination_prefix', 'next_hop', 'distance', 'comments')
        default_columns = ('pk', 'destination_prefix', 'next_hop', 'site', 'device', 'vrf')



from extras.plugins import PluginMenuButton, PluginMenuItem
from utilities.choices import ButtonColorChoices


staticroute_buttons = [
    PluginMenuButton(
        link='plugins:netbox_static_routes:staticroute_add',
        title='Add',
        icon_class='mdi mdi-plus-thick',
        color=ButtonColorChoices.GREEN
    )
]

menu_items = (
    PluginMenuItem(
        link='plugins:netbox_static_routes:staticroute_list',
        link_text='Static Routes',
        buttons=staticroute_buttons
    ),
)



from rest_framework import serializers
from ipam.api.serializers import NestedPrefixSerializer
from netbox.api.serializers import NetBoxModelSerializer, WritableNestedSerializer
from ..models import StaticRoute

class NestedStaticRouteSerializer(WritableNestedSerializer):
    url = serializers.HyperlinkedIdentityField(
        view_name='plugins-api:netbox_static_routes-api:staticroute-detail'
    )

    class Meta:
        model = StaticRoute
        fields = ('id', 'url', 'display', 'destination_prefix', 'next_hop', 'device', 'vrf')


class StaticRouteSerializer(NetBoxModelSerializer):
    url = serializers.HyperlinkedIdentityField(
        view_name='plugins-api:netbox_static_routes-api:staticroute-detail'
    )

    class Meta:
        model = StaticRoute
        fields = (
            'id', 'url', 'display', 'destination_prefix', 'next_hop', 'site', 'device', 'vrf', 'comments', 'tags', 'custom_fields', 'created', 'last_updated'
        )


#from django.contrib.postgres.fields import ArrayField
from django.db import models
from django.urls import reverse
from django.core.validators import MaxValueValidator, MinValueValidator
from django.contrib.postgres.fields import ArrayField
from netbox.models import NetBoxModel
from utilities.choices import ChoiceSet

# future use
class NextHopTypeChoices(ChoiceSet):
    key = 'StaticRoute.next_hop_type'

    NH_TYPE_IP_ADDRESS = ('ip_address', 'IP Address')
    NH_TYPE_INTERFACE = ('interface', 'Interface')
    NH_TYPE_VRF = ('vrf', 'VRF')

    CHOICES = [
        NH_TYPE_IP_ADDRESS,
        NH_TYPE_INTERFACE,
        NH_TYPE_VRF
    ]

class StaticRoute(NetBoxModel):
    site = models.ForeignKey(
        to='dcim.Site',
        on_delete=models.PROTECT,
        related_name="%(class)s_related",
        blank=True,
        null=True
    )
    device = models.ForeignKey(
        to='dcim.Device',
        on_delete=models.PROTECT,
        related_name='+',
        null=True,
    )
    vrf = models.ForeignKey(
        to='ipam.VRF',
        on_delete=models.PROTECT,
        related_name='+',
        blank=True,
        null=True,
        verbose_name='VRF'
    )
    destination_prefix = models.ForeignKey(
        to='ipam.Prefix',
        on_delete=models.PROTECT,
        null=True
    )
    next_hop = ArrayField(
        base_field=models.GenericIPAddressField(),
        null=True
    )
    distance = models.PositiveIntegerField(
        default=1,
        validators=[
            MinValueValidator(1),
            MaxValueValidator(255)
        ],
        verbose_name='Administrative distance'
    )
    comments = models.TextField(
        blank=True
    )

    class Meta:
        verbose_name_plural = 'Static Routes'
        ordering = ('device', 'vrf', 'destination_prefix', 'next_hop')
        unique_together = ['device', 'vrf', 'destination_prefix']

    def __str__(self):
        return f'{self.device}:{self.vrf}:{self.destination_prefix}'

    def get_absolute_url(self):
        return reverse('plugins:netbox_static_routes:staticroute', args=[self.pk])



from setuptools import find_packages, setup

setup(
    name = 'netbox-static-routes-plugin',
    version = '0.3', # would be nice to import the version from netbox_static_routes/version.py
    description = 'Manage static routes in Netbox',
    url = 'https://github.com/jbparrish17/netbox-static-routes',
    author = 'Joshua Parrish',
    license = 'Apache 2.0',
    install_requires = [],
    packages = find_packages(),
    include_package_data = True,
    package_data = {
        'netbox_static_routes_plugin': ['templates/*/*.html']
    },
    zip_safe = False
)



from netbox.views import generic
from . import filtersets, forms, models, tables

class StaticRouteView(generic.ObjectView):
    queryset = models.StaticRoute.objects.all()

class StaticRouteListView(generic.ObjectListView):
    queryset = models.StaticRoute.objects.all()
    table = tables.StaticRouteTable
    filterset = filtersets.StaticRouteFilterSet
    filterset_form = forms.StaticRouteFilterForm

class StaticRouteEditView(generic.ObjectEditView):
    queryset = models.StaticRoute.objects.all()
    form = forms.StaticRouteForm

class StaticRouteBulkEditView(generic.BulkEditView):
    queryset = models.StaticRoute.objects.all()
    table = tables.StaticRouteTable
    form = forms.StaticRouteBulkEditForm
    filterset = filtersets.StaticRouteFilterSet

class StaticRouteDeleteView(generic.ObjectDeleteView):
    queryset = models.StaticRoute.objects.all()

class StaticRouteBulkDeleteView(generic.BulkDeleteView):
    queryset = models.StaticRoute.objects.all()
    table = tables.StaticRouteTable




# Generated by Django 4.0.4 on 2022-05-30 17:44

import django.contrib.postgres.fields
import django.core.serializers.json
import django.core.validators
from django.db import migrations, models
import django.db.models.deletion
import taggit.managers


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('dcim', '0153_created_datetimefield'),
        ('extras', '0073_journalentry_tags_custom_fields'),
        ('ipam', '0057_created_datetimefield'),
    ]

    operations = [
        migrations.CreateModel(
            name='StaticRoute',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False)),
                ('created', models.DateTimeField(auto_now_add=True, null=True)),
                ('last_updated', models.DateTimeField(auto_now=True, null=True)),
                ('custom_field_data', models.JSONField(blank=True, default=dict, encoder=django.core.serializers.json.DjangoJSONEncoder)),
                ('next_hop', django.contrib.postgres.fields.ArrayField(base_field=models.GenericIPAddressField(), null=True, size=None)),
                ('distance', models.PositiveIntegerField(default=1, validators=[django.core.validators.MinValueValidator(1), django.core.validators.MaxValueValidator(255)])),
                ('comments', models.TextField(blank=True)),
                ('destination_prefix', models.ForeignKey(null=True, on_delete=django.db.models.deletion.PROTECT, to='ipam.prefix')),
                ('device', models.ForeignKey(null=True, on_delete=django.db.models.deletion.PROTECT, related_name='+', to='dcim.device')),
                ('site', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='%(class)s_related', to='dcim.site')),
                ('tags', taggit.managers.TaggableManager(through='extras.TaggedItem', to='extras.Tag')),
                ('vrf', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='+', to='ipam.vrf')),
            ],
            options={
                'verbose_name_plural': 'Static Routes',
                'ordering': ('device', 'vrf', 'destination_prefix', 'next_hop'),
                'unique_together': {('device', 'vrf', 'destination_prefix')},
            },
        ),
    ]
